#ifndef _PACKAGE_CPP
#define _PACKAGE_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <ShlObj.H>
#include <Stdio.H>
#include <Stdlib.H>

#include "../Resources/Resource.H"

#include "NSWFL.H"
#include "IndexDir.H"
#include "Package.H"
#include "Entry.H"
#include "CRC32.H"
#include "Common.H"

#include "../Dialogs/MainDlg.H"
#include "../Dialogs/OpenPassDlg.H"
#include "../Dialogs/ReadingDlg.H"
#include "../Dialogs/PackingDlg.H"
#include "../Dialogs/ExtractDlg.H"

#include "../Modules/NASCCL/NASCCL.H"
#include "../Modules/LZSS/LZSS_Compress.H"
#include "../Modules/LZSS/LZSS_Uncompress.H"

#include "../Modules/Rijndael/AES.H"
#include "../Modules/Rijndael/AESOpt.H"
#include "../Modules/Rijndael/Rijndael.H"

#include "../Modules/CCode/CCode.H"

#include "../Modules/BlowFish/BlowFish.H"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD dwPackageProgress = 0;
float fFileProgress = 0;
DWORD dwWorker_Thread_ID = 0;
HANDLE hWorker_Thread_Handle = NULL;

FILEPACKAGE AltFilePackage;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD UpdateFileCount(HANDLE hSource, NASCCL *MyNASCCL, DWORD dwFileCount)
{
	DWORD dwEncryptedFC = dwFileCount;
	DWORD dwBytesWritten = 0;

	//Skip past all of the header data that we dont want to overwrite.
	SetFilePointer(hSource, FILECOUNTPOS, NULL, FILE_BEGIN);

	//Write the new file count.
	MyNASCCL->Encode(&dwEncryptedFC, &dwEncryptedFC, sizeof(dwEncryptedFC));
	WriteFile(hSource, &dwEncryptedFC, sizeof(dwEncryptedFC), &dwBytesWritten, NULL);

	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool MakePackage(LPMKPKGPARAM lpMyMkPkgParam)
{
    unsigned char *sBlockBuffer1 = NULL;
    unsigned char *sBlockBuffer2 = NULL;
    
	char sExt[MAX_PATH + 1];
    char sFileName[MAX_PATH + 1];
    char sFullFileName[MAX_PATH + 1];
    char sLastDirName[MAX_PATH + 1];
    char sStatusText[1024];
    char sTempBuf[1024];
    char sThumbPrint[THUMB_PRINT_SZ + 1];

	DWORD dwCompatVersion = 0;
	DWORD dwCreationDate = 0;
	DWORD dwCreationTime = 0;
	DWORD dwOldFileCount = 0; //Only used if we are adding files to the archive.
	DWORD dwBytesRead = 0;
    DWORD dwBytesWritten = 0;
    DWORD dwCompressedFileSize = 0;
    DWORD dwCompSeekPos = 0;
    DWORD dwCurrentFPointer = 0;
    DWORD dwDeflSize = 0;
    DWORD dwBlockSize = -1;
    DWORD dwFileCount = 0;
    DWORD dwFileSize = 0;
	DWORD dwTotalBytesRead = 0;

	bool bTargetExist = false;

	FILETIME ftCreationTime;
    FILETIME ftLastWriteTime;

	HANDLE hfSource = NULL;
    HANDLE hfTarget = NULL;

	int iCompressResult = 0;
    int iBufferSize = DEFAULT_BUF_SZ;
    int iTemp = 0;

	long lChecksum1 = 0;
    long lChecksum2 = 0;

	unsigned char *sSourceBuf = NULL;
    unsigned char *sTargetBuf = NULL;
    unsigned long ulFinalCRC = 0;
    unsigned long ulLoopCount = 0;
    unsigned short usFileNameLen = 0;

    SIMPLECIPHER MySimpleCipher;
	NASCCL MyNASCCL;
	BlowFishCTX MyBlowFish;
	aes_ctx MyAes_Ctx;
	aes_ctx MyAes_CtxPerFile;
    FilePack MyFilePack;
	HS_LZSS_Compress MyLZSS_Compress;
	FILEPACKAGE MyFilePackage;

    memset(&MyNASCCL, 0, sizeof(MyNASCCL));
	memset(&MyFilePack, 0, sizeof(MyFilePack));
    memset(sTempBuf, 0, sizeof(sTempBuf));
    memset(&MyFilePackage, 0, sizeof(FILEPACKAGE));

	if(FileAccess(lpMyMkPkgParam->sTarName, FExist))
	{
		bTargetExist = true;
	}
	else bTargetExist = false;

	if(lpMyMkPkgParam->iOptions &ATTRIB_USEALTFILEPACK)
	{
		memcpy(&MyFilePackage, &AltFilePackage, sizeof(FILEPACKAGE));
	}
	else{
		MyFilePack.Initialize(&MyFilePackage);

		Set_Text(PackingActionText_hWnd, "Searching:");

		if(!MyFilePack.IndexDirectoryEx(lpMyMkPkgParam->sSrcFolder, true, PackingStatusText_hWnd, &MyFilePackage))
		{
			if(gdwCurrentJobType == JOB_TYPE_CANCEL)
			{
				return true;
			}
			else{
				sprintf(sStatusText, "An error occured while trying to index the directory:\r\n'%s'", lpMyMkPkgParam->sSrcFolder);
				MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
				return false;
			}
		}
	}

    GetLastDirectoryInPath(lpMyMkPkgParam->sSrcFolder, sLastDirName);

    //Sleep(1000);

    MyNASCCL.InitializeCryptographySet(lpMyMkPkgParam->sRawKey, lpMyMkPkgParam->iRawKeyLen, CRYPTMETHOD);

	if((lpMyMkPkgParam->iOptions &ATTRIB_ADDFILES) && bTargetExist)
	{
		hfTarget = CreateFile(
			lpMyMkPkgParam->sTarName,
			GENERIC_READ|GENERIC_WRITE,
			0,
			NULL,
			OPEN_EXISTING,
			FILE_ATTRIBUTE_NORMAL,
			NULL
		);
	}
	else{
		hfTarget = CreateFile(
			lpMyMkPkgParam->sTarName,
			GENERIC_WRITE,
			0,
			NULL,
			CREATE_ALWAYS,
			FILE_ATTRIBUTE_NORMAL,
			NULL
		);
	}

	if(hfTarget == INVALID_HANDLE_VALUE)
    {
        sprintf(sStatusText, "Failed to open write file:\r\n'%s'", lpMyMkPkgParam->sTarName);
        MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
        return false;
    }

	//Read or Write the header.
	if((lpMyMkPkgParam->iOptions &ATTRIB_ADDFILES) && bTargetExist)
	{
		//Read the Application Checksum.
		ReadFile(hfTarget, &lChecksum1, sizeof(lChecksum1), &dwBytesRead, NULL);

		//Read the Application Version Checksum.
		ReadFile(hfTarget, &dwCompatVersion, sizeof(dwCompatVersion), &dwBytesRead, NULL);

		if(dwCompatVersion != MakeChecksum(COMP_VERSION))
		{
			MessageBox(PackingDialog_hWnd, "The file you are trying to decrypt was encrypted using an incompatible version.", gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
	        return false;
		}

		//Read the Password Checksum.
		ReadFile(hfTarget, &lChecksum2, sizeof(lChecksum2), &dwBytesRead, NULL);
		MyNASCCL.Decode(&lChecksum2, &lChecksum2, sizeof(lChecksum2));

		//Read the Package Attributes.
		ReadFile(hfTarget, &iTemp, sizeof(iTemp), &dwBytesRead, NULL);
		MyNASCCL.Decode(&iTemp, &lpMyMkPkgParam->iPkgAttribs, sizeof(iTemp));

		//Read the Cryptography Algorithm.
		ReadFile(hfTarget, &iTemp, sizeof(iTemp), &dwBytesWritten, NULL);
		MyNASCCL.Decode(&iTemp, &giCryptAlgo, sizeof(iTemp));

		//Read the Julian Creation Date.
		ReadFile(hfTarget, &dwCreationDate, sizeof(dwCreationDate), &dwBytesRead, NULL);
		MyNASCCL.Decode(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));

		//Read the Julian Creation Time.
		ReadFile(hfTarget, &dwCreationTime, sizeof(dwCreationTime), &dwBytesRead, NULL);
		MyNASCCL.Decode(&dwCreationTime, &dwCreationTime, sizeof(dwCreationTime));

		//Read the File thumb print or serial number.
		ReadFile(hfTarget, sThumbPrint, THUMB_PRINT_SZ, &dwBytesRead, NULL);
		MyNASCCL.Decode(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);

		//Skip past the Reserved header space.
		SetFilePointer(hfTarget, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

		//Read the File count.
		ReadFile(hfTarget, &dwFileCount, sizeof(DWORD), &dwBytesRead, NULL);
		MyNASCCL.Decode(&dwFileCount, &dwFileCount, sizeof(DWORD));

		dwOldFileCount = dwFileCount;

		dwFileCount = (dwFileCount + MyFilePackage.dwFileCount);

		UpdateFileCount(hfTarget, &MyNASCCL, dwFileCount);
		
		SetFilePointer(hfTarget, 0, NULL, FILE_END);
	}
	else{
		SYSTEMTIME stCreation;

		GetSystemTime(&stCreation);
		dwCreationDate = DateToLong(stCreation.wYear, stCreation.wMonth, stCreation.wDay);
		dwCreationTime = TimeToLong(stCreation.wHour, stCreation.wMinute, stCreation.wSecond);
	
		GenerateKey(THUMB_PRINT_SZ, GenerateSeed(), GKNONREPETITION | GKBROADKEY, sThumbPrint);

		//Write the Application Checksum.
		lChecksum1 = MakeChecksum(lpMyMkPkgParam->sRawKey);
		WriteFile(hfTarget, &lChecksum1, sizeof(lChecksum1), &dwBytesWritten, NULL);

		//Write the Application Version Checksum.
		dwCompatVersion = MakeChecksum(COMP_VERSION);
		WriteFile(hfTarget, &dwCompatVersion, sizeof(dwCompatVersion), &dwBytesWritten, NULL);

		//Write the Password Checksum.
		lChecksum2 = HARDCHECKSUM;
		MyNASCCL.Encode(&lChecksum2, &lChecksum2, sizeof(lChecksum2));
		WriteFile(hfTarget, &lChecksum2, sizeof(lChecksum2), &dwBytesWritten, NULL);

		//Write the Package Attributes.
		MyNASCCL.Encode(&lpMyMkPkgParam->iPkgAttribs, &iTemp, sizeof(int));
		WriteFile(hfTarget, &iTemp, sizeof(int), &dwBytesWritten, NULL);

		//Write the Cryptography Algorithm
		MyNASCCL.Encode(&giCryptAlgo, &iTemp, sizeof(giCryptAlgo));
		WriteFile(hfTarget, &iTemp, sizeof(giCryptAlgo), &dwBytesWritten, NULL);

		//Write the Julian Creation Date.
		MyNASCCL.Encode(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));
		WriteFile(hfTarget, &dwCreationDate, sizeof(dwCreationDate), &dwBytesWritten, NULL);

		//Write the Julian Creation Time.
		MyNASCCL.Encode(&dwCreationTime, &dwCreationTime, sizeof(dwCreationDate));
		WriteFile(hfTarget, &dwCreationTime, sizeof(dwCreationDate), &dwBytesWritten, NULL);

		//Write the File thumb print or serial number.
		MyNASCCL.Encode(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);
		WriteFile(hfTarget, sThumbPrint, THUMB_PRINT_SZ, &dwBytesWritten, NULL);

		//Skip past the Reserved header space.
		SetFilePointer(hfTarget, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

		//Write the File count.
		MyNASCCL.Encode(&MyFilePackage.dwFileCount, &dwFileCount, sizeof(dwFileCount));
		WriteFile(hfTarget, &dwFileCount, sizeof(dwFileCount), &dwBytesWritten, NULL);
	}

    SendMessage(PackingStatusBar_hWnd, PBM_SETRANGE32, 0, (LPARAM) MyFilePackage.dwFileCount);

    sSourceBuf = (unsigned char *) calloc(iBufferSize + 1, sizeof(char));
    sTargetBuf = (unsigned char *) calloc((iBufferSize + EXTRA_SIZE) + 1, sizeof(char));

    Set_Text(PackingActionText_hWnd, "Adding:");

	if(giCryptAlgo == CRYPT_ALGO_SIMPLE)
	{
		char sSimpleKey[SIMPLE_KEY_LEN + 1];
		HashKey(lpMyMkPkgParam->sRawKey, sSimpleKey, lpMyMkPkgParam->iRawKeyLen, SIMPLE_KEY_LEN);

		memset(&MySimpleCipher, 0, sizeof(MySimpleCipher));
	
		InitializeCryptography(&MySimpleCipher, sSimpleKey, SIMPLE_KEY_LEN);

		dwBlockSize = -1;
	}

	if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
	{
		char sRijndaelKey[RIJNDAEL_KEY_LEN + 1];
		HashKey(lpMyMkPkgParam->sRawKey, sRijndaelKey, lpMyMkPkgParam->iRawKeyLen, RIJNDAEL_KEY_LEN);

		memset(&MyAes_Ctx, 0, sizeof(MyAes_Ctx));
	
		aes_enc_key((unsigned char *)sRijndaelKey, RIJNDAEL_KEY_LEN, &MyAes_Ctx);
		dwBlockSize = 32;

		sBlockBuffer1 = (unsigned char *) calloc(dwBlockSize + 1, sizeof(unsigned char));
		sBlockBuffer2 = (unsigned char *) calloc(dwBlockSize + 1, sizeof(unsigned char));
	}

	if(giCryptAlgo == CRYPT_ALGO_BLOWFISH)
	{
		char sBlowFishKey[BLOWFISH_KEY_LEN + 1];
		HashKey(lpMyMkPkgParam->sRawKey, sBlowFishKey, lpMyMkPkgParam->iRawKeyLen, BLOWFISH_KEY_LEN);
			
		memset(&MyBlowFish, 0, sizeof(MyBlowFish));
	
		Blowfish_Init(&MyBlowFish, (unsigned char *)sBlowFishKey, BLOWFISH_KEY_LEN);
		dwBlockSize = 8; //Two DWORDS's

		sBlockBuffer1 = (unsigned char *) calloc(dwBlockSize + 1, sizeof(unsigned char));
		sBlockBuffer2 = (unsigned char *) calloc(dwBlockSize + 1, sizeof(unsigned char));
	}

	if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_COMPRESSION)
    {
        MyLZSS_Compress.SetupCompressionLevel(giCompressLevel);
    }

	while(ulLoopCount < MyFilePackage.dwFileCount)
    {
        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
			if((lpMyMkPkgParam->iOptions &ATTRIB_ADDFILES) && !bTargetExist)
			{
				UpdateFileCount(hfTarget, &MyNASCCL, dwOldFileCount + ulLoopCount);
			}
            Sleep(1000);
            break;
        }

		strcpy(sFullFileName, lpMyMkPkgParam->sSrcFolder);
        strcat(sFullFileName, MyFilePackage.FileNames[ulLoopCount]);

        if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_SAVEPATHS)
        {
            strcpy(sTempBuf, sLastDirName);
            strcat(sTempBuf, MyFilePackage.FileNames[ulLoopCount]);
        }
        else{
            _splitpath(MyFilePackage.FileNames[ulLoopCount], NULL, NULL, sFileName, sExt);
            _makepath(sTempBuf, NULL, NULL, sFileName, sExt);
        }

        usFileNameLen = strlen(sTempBuf);

        WriteFile(hfTarget, &usFileNameLen, sizeof(unsigned short), &dwBytesWritten, NULL);
		MyNASCCL.Encode(sTempBuf, sTempBuf, usFileNameLen);
        WriteFile(hfTarget, sTempBuf, usFileNameLen, &dwBytesWritten, NULL);

        SetStatusFileName(PackingStatusText_hWnd, MyFilePackage.FileNames[ulLoopCount]);

        hfSource = CreateFile(
            sFullFileName,
            GENERIC_READ,
            FILE_SHARE_READ,
            NULL,
            OPEN_EXISTING,
            FILE_ATTRIBUTE_NORMAL,
            NULL
        );

        if(hfSource == INVALID_HANDLE_VALUE)
        {
            sprintf(sStatusText, "Failed to open read file:\r\n'%s'", sFullFileName);
            MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

            CloseHandle(hfTarget);
            free(sSourceBuf);
            free(sTargetBuf);

			//Free memory for the RIJNDAEL algo.
			if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
			{
				free(sBlockBuffer1);
				free(sBlockBuffer2);
			}
			else if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL){
				//Free memory used by the simple algo.
				DestroyCryptography(&MySimpleCipher);
			}
			if((lpMyMkPkgParam->iOptions &ATTRIB_ADDFILES) && !bTargetExist)
			{
				UpdateFileCount(hfTarget, &MyNASCCL, dwOldFileCount + ulLoopCount - 1);
			}
            return false;
        }

		if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
		{
			memcpy(&MyAes_CtxPerFile, &MyAes_Ctx, sizeof(MyAes_Ctx));
		}

		dwFileSize = GetFileSize(hfSource, NULL);
        WriteFile(hfTarget, &dwFileSize, sizeof(DWORD), &dwBytesWritten, NULL);

		fFileProgress = 0;
		dwTotalBytesRead = 0;

		SendMessage(FileStatusBar_hWnd, PBM_SETPOS, 0, (LPARAM) 0);

        dwCompSeekPos = SetFilePointer(hfTarget, 0, NULL, FILE_CURRENT);
        // To make room for the compressed size
		SetFilePointer(hfTarget, sizeof(DWORD), NULL, FILE_CURRENT);
        //WriteFile(hfTarget, &dwFileSize, sizeof(DWORD), &dwBytesWritten, NULL);

        if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_DATETIME)
        {
            GetFileTime(hfSource, &ftCreationTime, NULL, &ftLastWriteTime);
            WriteFile(hfTarget, &ftCreationTime, sizeof(FILETIME), &dwBytesWritten, NULL);
            WriteFile(hfTarget, &ftLastWriteTime, sizeof(FILETIME), &dwBytesWritten, NULL);
        }

        if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_CRCCHECK)
        {
        	ulFinalCRC = 0xffffffff;
        }

        bool bCompression = true;

        do{
			if(!(lpMyMkPkgParam->iOptions &ATTRIB_ADDFILES))
			{
				if(gdwCurrentJobType == JOB_TYPE_CANCEL)
				{
					//No need to update the file count here. We will be deleting the archive.
					Sleep(1000);
					break;
				}
			}

            ReadFile(hfSource, sSourceBuf, iBufferSize, &dwBytesRead, NULL);
			dwTotalBytesRead = (dwTotalBytesRead + dwBytesRead);
			
			fFileProgress = ((float)((((float)dwTotalBytesRead) / ((float)dwFileSize)) * 100));

            if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_CRCCHECK)
            {
                ulFinalCRC = PartialCRC(ulFinalCRC, sSourceBuf, dwBytesRead);
            }
			if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_COMPRESSION)
            {
        		iCompressResult = MyLZSS_Compress.Compress(sSourceBuf, sTargetBuf, dwBytesRead, &dwDeflSize);
            }
            else iCompressResult = -1;

            if(iCompressResult != HS_LZSS_E_OK)
            {
                memcpy(sTargetBuf, sSourceBuf, dwBytesRead);
                dwDeflSize = dwBytesRead;
                bCompression = false;
                WriteFile(hfTarget, &bCompression, sizeof(bool), &dwBytesWritten, NULL);
            }
            else{
                bCompression = true;
                WriteFile(hfTarget, &bCompression, sizeof(bool), &dwBytesWritten, NULL);
            }

            sTargetBuf[dwDeflSize] = '\0';

			WriteFile(hfTarget, &dwBytesRead, sizeof(DWORD), &dwBytesWritten, NULL);
			WriteFile(hfTarget, &dwDeflSize, sizeof(DWORD), &dwBytesWritten, NULL);

			if(giCryptAlgo == CRYPT_ALGO_NASCCL)
			{
				MyNASCCL.Encode(sTargetBuf, sTargetBuf, dwDeflSize);
				WriteFile(hfTarget, sTargetBuf, dwDeflSize, &dwBytesWritten, NULL);
			}
			else if(giCryptAlgo == CRYPT_ALGO_SIMPLE){
				Encode(&MySimpleCipher, (char *)sTargetBuf, (char *)sTargetBuf, dwDeflSize);
				WriteFile(hfTarget, sTargetBuf, dwDeflSize, &dwBytesWritten, NULL);
			}
			else if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL) {
				
				DWORD dwBlockGap = dwBlockSize - (dwDeflSize % dwBlockSize);

				WriteFile(hfTarget, &dwBlockGap, sizeof(DWORD), &dwBytesWritten, NULL);

				DWORD dwRPos = 0;
				DWORD dwWPos = 0;
				DWORD dwFill = 0;

				while(dwRPos < dwDeflSize)
				{
					dwWPos = 0;
					while(dwWPos < dwBlockSize && dwRPos < dwDeflSize)
						sBlockBuffer1[dwWPos++] = sTargetBuf[dwRPos++];

					dwFill = dwWPos;
					while(dwFill < dwBlockSize)
						sBlockBuffer1[dwFill++] = '*';
				
					//memcpy(sBlockBuffer2, sBlockBuffer1, dwBlockSize);

					aes_enc_blk((unsigned char *)sBlockBuffer1, (unsigned char *)sBlockBuffer2, &MyAes_CtxPerFile);

					WriteFile(hfTarget, sBlockBuffer2, dwFill, &dwBytesWritten, NULL);
				}
			}
			else if(giCryptAlgo == CRYPT_ALGO_BLOWFISH) {
				DWORD dwBlockGap = dwBlockSize - (dwDeflSize % dwBlockSize);
				DWORD dwL = 0;
				DWORD dwR = 0;

				WriteFile(hfTarget, &dwBlockGap, sizeof(DWORD), &dwBytesWritten, NULL);

				DWORD dwRPos = 0;
				DWORD dwWPos = 0;
				DWORD dwFill = 0;

				while(dwRPos < dwDeflSize)
				{
					dwWPos = 0;
				    while(dwWPos < sizeof(DWORD) && dwRPos < dwDeflSize)
						((char *)&dwL)[dwWPos++] = sTargetBuf[dwRPos++];

					dwFill = dwWPos;
					while(dwFill < sizeof(DWORD))
						((char *)&dwL)[dwFill++] = '0';
				
					dwWPos = 0;
					while(dwWPos < sizeof(DWORD) && dwRPos < dwDeflSize)
						((char *)&dwR)[dwWPos++] = sTargetBuf[dwRPos++];

					dwFill = dwWPos;
					while(dwFill < sizeof(DWORD))
						((char *)&dwR)[dwFill++] = '0';
				
					Blowfish_Encrypt(&MyBlowFish, &dwL, &dwR);

					WriteFile(hfTarget, &dwL, sizeof(DWORD), &dwBytesWritten, NULL);
					WriteFile(hfTarget, &dwR, sizeof(DWORD), &dwBytesWritten, NULL);
				}
			}
        } while(dwBytesRead == iBufferSize);

		fFileProgress = 100;
		SendMessage(FileStatusBar_hWnd, PBM_SETPOS, 100, (LPARAM) 0);
		Sleep(1);

		if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_CRCCHECK)
        {
            ulFinalCRC = (ulFinalCRC ^ 0xffffffff);
        }

        dwCurrentFPointer = SetFilePointer(hfTarget, 0, NULL, FILE_CURRENT);
        dwCompressedFileSize = ((dwCurrentFPointer - dwCompSeekPos) - sizeof(DWORD));
        SetFilePointer(hfTarget, dwCompSeekPos, NULL, FILE_BEGIN);
        WriteFile(hfTarget, &dwCompressedFileSize, sizeof(DWORD), &dwBytesWritten, NULL);
        SetFilePointer(hfTarget, dwCurrentFPointer, NULL, FILE_BEGIN);

        if(lpMyMkPkgParam->iPkgAttribs &ATTRIB_CRCCHECK)
        {
            WriteFile(hfTarget, &ulFinalCRC, sizeof(DWORD), &dwBytesWritten, NULL);
        }

        CloseHandle(hfSource);

        ulLoopCount++;
        dwPackageProgress = ulLoopCount;
	}

    free(sSourceBuf);
    free(sTargetBuf);

	//Free memory for the RIJNDAEL algo.
	if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
	{
		free(sBlockBuffer1);
		free(sBlockBuffer2);
	}
	else if(giCryptAlgo == CRYPT_ALGO_SIMPLE){
		//Free memory used by the simple algo.
		DestroyCryptography(&MySimpleCipher);
	}

	MyFilePack.FreeMemory(&MyFilePackage);

    CloseHandle(hfTarget);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool BreakPackage(char *SrcName, char *TarFolder, char *sRawKey, int RawKeyLen)
{
    char *sBlockBuffer1 = NULL;
    char *sBlockBuffer2 = NULL;

	char sFullFileName[MAX_PATH + 1];
    char sPartFileName[MAX_PATH + 1];
    char sStatusText[1024];
    char sTempBuf[1024];
    char sThumbPrint[THUMB_PRINT_SZ + 1];

	DWORD dwCompatVersion = 0;
	DWORD dwCreationDate = 0;
	DWORD dwCreationTime = 0;
	DWORD dwBytesRead = 0;
    DWORD dwBytesWritten = 0;
    DWORD dwCompressedFileSize = 0;
    DWORD dwDeflSize = 0;
    DWORD dwBlockSize = -1;
    DWORD dwFileCount = 0;
    DWORD dwFileSize = 0;
    DWORD dwInflSize = 0;
    DWORD dwOrigSize = 0;
	DWORD dwTotalBytesRead = 0;

	FILETIME ftCreationTime;
    FILETIME ftLastWriteTime;

	HANDLE hfSource = NULL;
    HANDLE hfTarget = NULL;

	int iCompressResult = 0;
    int iBufferSize = DEFAULT_BUF_SZ;
    int iTemp = 0;
    int iPkgAttribs = 0;
    int iLoopFileCountTo = 0;
    int iLoopFileCount = 0;

	long lChecksum = 0;
    long lReadChecksum1 = 0;
    long lReadChecksum2 = 0;

	unsigned char *sSourceBuf = NULL;
    unsigned char *sTargetBuf = NULL;

	unsigned long ulFinalCRC = 0;
    unsigned long ulLoopCount = 0;
    unsigned long ulReadCRC = 0;

	unsigned short usFileNameLen = 0;

    SIMPLECIPHER MySimpleCipher;
    NASCCL MyNASCCL;
	BlowFishCTX MyBlowFish;
	aes_ctx MyAes_Ctx;
	aes_ctx MyAes_CtxPerFile;
	HS_LZSS_Uncompress MyLZSS_Uncompress;

    memset(&MyNASCCL, 0, sizeof(MyNASCCL));
    memset(sTempBuf, 0, sizeof(sTempBuf));

	MyNASCCL.InitializeCryptographySet(sRawKey, RawKeyLen, CRYPTMETHOD);

    lChecksum = MakeChecksum(sRawKey);

    hfSource = CreateFile(
        SrcName,
        GENERIC_READ,
        FILE_SHARE_READ,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
        sprintf(sStatusText, "Failed to open read file:\r\n'%s'.", SrcName);
        MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
        return false;
    }

	//Read the header.
	//------------------------------------------------------------------------------------------------

	//Read the Application Checksum.
	ReadFile(hfSource, &lReadChecksum1, sizeof(lReadChecksum1), &dwBytesRead, NULL);

	//Read the Application Version Checksum.
	ReadFile(hfSource, &dwCompatVersion, sizeof(dwCompatVersion), &dwBytesRead, NULL);

	if(dwCompatVersion != MakeChecksum(COMP_VERSION))
	{
		MessageBox(PackingDialog_hWnd, "The file you are trying to decrypt was encrypted using an incompatible version.", gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
	    return false;
	}

	//Read the Password Checksum.
	ReadFile(hfSource, &lReadChecksum2, sizeof(lReadChecksum2), &dwBytesRead, NULL);
    MyNASCCL.Decode(&lReadChecksum2, &lReadChecksum2, sizeof(lReadChecksum2));

    if(lReadChecksum1 != lChecksum || lReadChecksum2 != HARDCHECKSUM)
    {
        MessageBox(PackingDialog_hWnd, "Wrong key, Can't decrypt.", gsTitleCaption, MB_ICONINFORMATION|MB_OK|MB_APPLMODAL);
        CloseHandle(hfSource);
        return false;
    }

	//Read the Package Attributes.
    ReadFile(hfSource, &iPkgAttribs, sizeof(iPkgAttribs), &dwBytesRead, NULL);
    MyNASCCL.Decode(&iPkgAttribs, &iPkgAttribs, sizeof(iPkgAttribs));

	//Read the Cryptography Algorithm.
	ReadFile(hfSource, &giCryptAlgo, sizeof(giCryptAlgo), &dwBytesRead, NULL);
    MyNASCCL.Decode(&giCryptAlgo, &giCryptAlgo, sizeof(giCryptAlgo));

	//Read the Julian Creation Date.
	ReadFile(hfSource, &dwCreationDate, sizeof(dwCreationDate), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));

	//Read the Julian Creation Time.
	ReadFile(hfSource, &dwCreationTime, sizeof(dwCreationTime), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwCreationTime, &dwCreationTime, sizeof(dwCreationTime));

	//Read the File thumb print or serial number.
	ReadFile(hfSource, sThumbPrint, THUMB_PRINT_SZ, &dwBytesRead, NULL);
    MyNASCCL.Decode(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);

	//Skip past the Reserved header space.
	SetFilePointer(hfSource, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

	//Read the File count.
	ReadFile(hfSource, &dwFileCount, sizeof(dwFileCount), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwFileCount, &dwFileCount, sizeof(dwFileCount));

	//------------------------------------------------------------------------------------------------

	if(gdwCurrentJobType == JOB_TYPE_EXTRACTSEL)
	{
		iLoopFileCountTo = ListView_GetSelectedCount(FileList_hWnd);
		SendMessage(PackingStatusBar_hWnd, PBM_SETRANGE32, 0, (LPARAM) iLoopFileCountTo);
	}
	else{
		iLoopFileCountTo = dwFileCount;
		SendMessage(PackingStatusBar_hWnd, PBM_SETRANGE32, 0, (LPARAM) iLoopFileCountTo);
	}

    dwPackageProgress = 0;

    sSourceBuf = (unsigned char *) calloc((iBufferSize + EXTRA_SIZE) + 1, sizeof(char));
    sTargetBuf = (unsigned char *) calloc((iBufferSize + EXTRA_SIZE) + 1, sizeof(char));

	if(giCryptAlgo == CRYPT_ALGO_SIMPLE)
	{
		char sSimpleKey[SIMPLE_KEY_LEN + 1];
		HashKey(sRawKey, sSimpleKey, RawKeyLen, SIMPLE_KEY_LEN);

		memset(&MySimpleCipher, 0, sizeof(MySimpleCipher));
	
		InitializeCryptography(&MySimpleCipher, sSimpleKey, SIMPLE_KEY_LEN);

		dwBlockSize = -1;
	}

	if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
	{
		char sRijndaelKey[RIJNDAEL_KEY_LEN + 1];
		HashKey(sRawKey, sRijndaelKey, RawKeyLen, RIJNDAEL_KEY_LEN);

		memset(&MyAes_Ctx, 0, sizeof(MyAes_Ctx));
	
		aes_dec_key((unsigned char *)sRijndaelKey, RIJNDAEL_KEY_LEN, &MyAes_Ctx);
		dwBlockSize = 32;

		sBlockBuffer1 = (char *) calloc(dwBlockSize + 1, sizeof(char));
		sBlockBuffer2 = (char *) calloc(dwBlockSize + 1, sizeof(char));
	}

	if(giCryptAlgo == CRYPT_ALGO_BLOWFISH)
	{
		char sBlowFishKey[BLOWFISH_KEY_LEN + 1];
		HashKey(sRawKey, sBlowFishKey, RawKeyLen, BLOWFISH_KEY_LEN);
			
		memset(&MyBlowFish, 0, sizeof(MyBlowFish));
	
		Blowfish_Init(&MyBlowFish, (unsigned char *)sBlowFishKey, BLOWFISH_KEY_LEN);
		dwBlockSize = 8;
	}

	//--------------------------------------------------------------------------------

    Set_Text(PackingActionText_hWnd, "Extracting:");

    int iLVItemPos = -1;

	DWORD dwFilePos = 0;

	while(iLoopFileCount < iLoopFileCountTo)
	{
		dwTotalBytesRead = 0;

        if(gdwCurrentJobType == JOB_TYPE_EXTRACTSEL)
        {
			DWORD dwTempfPos = 0;

			while((iLVItemPos = ListView_GetNextItem(FileList_hWnd, iLVItemPos, LVNI_SELECTED)) != -1)
            {
                ListView_GetItemText(FileList_hWnd, iLVItemPos, 9, sTempBuf, 255);
	            SetFilePointer(hfSource, atol(sTempBuf), NULL, FILE_BEGIN);
				break;
            }

			if(iLVItemPos == -1)
			{
				break;
			}
		}
		
	    ReadFile(hfSource, &usFileNameLen, sizeof(unsigned short), &dwBytesRead, NULL);
		if(dwBytesRead == 0)
		{
			//This is where this loop ends.
			break;
		}

		if(usFileNameLen <= 0 || usFileNameLen > 255)
        {
            sprintf(sStatusText, "Corrupt data file detected. Error code: %X", usFileNameLen);
            MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
            break;
        }

        ReadFile(hfSource, sPartFileName, usFileNameLen, &dwBytesRead, NULL);
        MyNASCCL.Decode(sPartFileName, sPartFileName, usFileNameLen);
    	sPartFileName[usFileNameLen] = '\0';
        ReadFile(hfSource, &dwFileSize, sizeof(DWORD), &dwBytesRead, NULL);
        ReadFile(hfSource, &dwCompressedFileSize, sizeof(DWORD), &dwBytesRead, NULL);

		if(iPkgAttribs &ATTRIB_DATETIME)
        {
            ReadFile(hfSource, &ftCreationTime, sizeof(FILETIME), &dwBytesRead, NULL);
            ReadFile(hfSource, &ftLastWriteTime, sizeof(FILETIME), &dwBytesRead, NULL);
        }

        strcpy(sFullFileName, TarFolder);
        strcat(sFullFileName, sPartFileName);

		if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Sleep(1000);
            break;
        }

        SetStatusFileName(PackingStatusText_hWnd, sPartFileName);

        CreateFolderStructure(sFullFileName);

        hfTarget = CreateFile(
            sFullFileName,
            GENERIC_WRITE,
            0,
            NULL,
            CREATE_ALWAYS,
            FILE_ATTRIBUTE_NORMAL,
            NULL
        );

		fFileProgress = 0;
		SendMessage(FileStatusBar_hWnd, PBM_SETPOS, 0, (LPARAM) 0);

		if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
		{
			memcpy(&MyAes_CtxPerFile, &MyAes_Ctx, sizeof(MyAes_Ctx));
		}

		if(iPkgAttribs &ATTRIB_CRCCHECK)
        {
        	ulFinalCRC = 0xffffffff;
        }

        bool bCompression = true;
		DWORD dwBlockGap = 0;
		DWORD dwAdjustedSize = 0;
		
		do{
			//dwCurrentFPOS = SetFilePointer(hfSource, 0, NULL, FILE_CURRENT);
			//MessageBox(GetActiveWindow(), IntChar(dwCurrentFPOS), sFullFileName, NULL);

			ReadFile(hfSource, &bCompression, sizeof(bool), &dwBytesRead, NULL);
            ReadFile(hfSource, &dwOrigSize, sizeof(DWORD), &dwBytesRead, NULL);
            ReadFile(hfSource, &dwDeflSize, sizeof(DWORD), &dwBytesRead, NULL);

			dwTotalBytesRead = (dwTotalBytesRead + dwOrigSize);
			
			fFileProgress = ((float)((((float)dwTotalBytesRead) / ((float)dwFileSize)) * 100));

			if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL || giCryptAlgo == CRYPT_ALGO_BLOWFISH)
			{
				ReadFile(hfSource, &dwBlockGap, sizeof(DWORD), &dwBytesRead, NULL);
				if(dwBlockGap == dwBlockSize)
				{
					dwBlockGap = 0;
				}
			}

			dwAdjustedSize = (dwDeflSize + dwBlockGap);

			ReadFile(hfSource, sSourceBuf, dwAdjustedSize, &dwBytesRead, NULL);

			sSourceBuf[dwBytesRead] = 0;

            dwInflSize = (iBufferSize + EXTRA_SIZE);

			if(giCryptAlgo == CRYPT_ALGO_NASCCL)
			{
				MyNASCCL.Decode(sSourceBuf, sSourceBuf, dwDeflSize);
			}
			else if(giCryptAlgo == CRYPT_ALGO_SIMPLE){
				Decode(&MySimpleCipher, (char *)sSourceBuf, (char *)sSourceBuf, dwDeflSize);
			}
			else if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL){
				DWORD dwRPos = 0;
				DWORD dwWPos = 0;

				DWORD dwCypRPos = 0;
				DWORD dwCypWPos = 0;

				while(dwRPos < dwAdjustedSize)
				{
					dwWPos = 0;
					while(dwWPos < dwBlockSize)
						sBlockBuffer1[dwWPos++] = sSourceBuf[dwRPos++];

					//memcpy(sBlockBuffer2, sBlockBuffer1, dwBlockSize);

					aes_dec_blk((unsigned char *)sBlockBuffer1, (unsigned char *)sBlockBuffer2, &MyAes_CtxPerFile);

					dwCypRPos = 0;
					while(dwCypRPos < dwWPos)					
						sSourceBuf[dwCypWPos++] = sBlockBuffer2[dwCypRPos++];
				}
				
				sSourceBuf[dwCypWPos - dwBlockGap] = '\0';
				dwBytesRead = (dwBytesRead - dwBlockGap);
			}
			else if(giCryptAlgo == CRYPT_ALGO_BLOWFISH) {
				DWORD dwL = 0;
				DWORD dwR = 0;

				DWORD dwRPos = 0;
				DWORD dwWPos = 0;

				DWORD dwCypRPos = 0;
				DWORD dwCypWPos = 0;

				while(dwRPos < dwAdjustedSize)
				{
					dwWPos = 0;
					while(dwWPos < sizeof(DWORD))
						((char *)&dwL)[dwWPos++] = sSourceBuf[dwRPos++];

					dwWPos = 0;
					while(dwWPos < sizeof(DWORD))
						((char *)&dwR)[dwWPos++] = sSourceBuf[dwRPos++];

					Blowfish_Decrypt(&MyBlowFish, &dwL, &dwR);

					dwCypRPos = 0;
					while(dwCypRPos < sizeof(DWORD))					
						sSourceBuf[dwCypWPos++] = ((char *)&dwL)[dwCypRPos++];

					dwCypRPos = 0;
					while(dwCypRPos < sizeof(DWORD))					
						sSourceBuf[dwCypWPos++] = ((char *)&dwR)[dwCypRPos++];
				}
				
				sSourceBuf[dwCypWPos - dwBlockGap] = '\0';
				dwBytesRead = (dwBytesRead - dwBlockGap);
			}

			if(bCompression && (iPkgAttribs &ATTRIB_COMPRESSION))
            {
        		iCompressResult = MyLZSS_Uncompress.GetUncompressedSize(sSourceBuf, &dwInflSize);
                MyLZSS_Uncompress.Uncompress(sSourceBuf, sTargetBuf);
                if(iPkgAttribs &ATTRIB_CRCCHECK)
                {
                    ulFinalCRC = PartialCRC(ulFinalCRC, sTargetBuf, dwInflSize);
                }
				WriteFile(hfTarget, sTargetBuf, dwInflSize, &dwBytesWritten, NULL);
            }
            else {
                if(iPkgAttribs &ATTRIB_CRCCHECK)
                {
                    ulFinalCRC = PartialCRC(ulFinalCRC, sSourceBuf, dwBytesRead);
                }
				WriteFile(hfTarget, sSourceBuf, dwBytesRead, &dwBytesWritten, NULL);
            }
        } while(dwOrigSize == iBufferSize);

		fFileProgress = 100;
		SendMessage(FileStatusBar_hWnd, PBM_SETPOS, 100, (LPARAM) 0);

        if(iPkgAttribs &ATTRIB_CRCCHECK)
        {
            ulFinalCRC = (ulFinalCRC ^ 0xffffffff);
            ReadFile(hfSource, &ulReadCRC, sizeof(unsigned long), &dwBytesRead, NULL);

            if(ulFinalCRC != ulReadCRC)
            {
                sprintf(sStatusText, "Bad CRC %X should be %X\r\n%s", ulFinalCRC, ulReadCRC, sPartFileName);
                MessageBox(PackingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
                break;
            }
        }

        if(iPkgAttribs &ATTRIB_DATETIME)
        {
            SetFileTime(hfTarget, &ftCreationTime, NULL, &ftLastWriteTime);
        }

        CloseHandle(hfTarget);

        //ReadFile(hfSource, &usFileNameLen, sizeof(unsigned short), &dwBytesRead, NULL);
        dwPackageProgress++;

		iLoopFileCount++;
	}

    //--------------------------------------------------------------------------------

    free(sSourceBuf);
    free(sTargetBuf);
	
	//Free memory for the RIJNDAEL algo.
	if(giCryptAlgo == CRYPT_ALGO_RIJNDAEL)
	{
		free(sBlockBuffer1);
		free(sBlockBuffer2);
	}
	else if(giCryptAlgo == CRYPT_ALGO_SIMPLE){
		//Free memory used by the simple algo.
		DestroyCryptography(&MySimpleCipher);
	}

    CloseHandle(hfSource);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool PopGrid(char *SrcName, char *sRawKey, int RawKeyLen)
{
	int iImageIndex = -1;
    int iIconSCount = 0;

	char sDirectory[MAX_PATH + 1];
    char sFileName[MAX_PATH + 1];
    char sExt[MAX_PATH + 1];
    char sDescr[1024 + 1];

	char sPartFileName[MAX_PATH + 1];
    char sStatusText[1024];
    char sTempBuf[1024];
    char TempBuf0[1024];
    char TempBuf1[1024];
    char sThumbPrint[THUMB_PRINT_SZ + 1];

	double TotalFileSizes = 0;

	DWORD dwCompatVersion = 0;
	DWORD dwCreationDate = 0;
	DWORD dwCreationTime = 0;
	DWORD dwBytesRead = 0;
    DWORD dwCompressedFileSize = 0;
    DWORD dwFilePosition = 0;
    DWORD dwFileCount = 0;
    DWORD dwFileCountLoop = 0;
    DWORD dwFileSize = 0;

	FILETIME ftCreationTime;
    FILETIME ftLastWriteTime;

	HANDLE hfSource = NULL;

	int iPkgAttribs = 0;

	long lChecksum = 0;
    long lReadChecksum1 = 0;
    long lReadChecksum2 = 0;

	unsigned long ulReadCRC = 0;
    unsigned short usFileNameLen = 0;

    NASCCL MyNASCCL;
    LV_ITEM MyLVItem;

    memset(&MyLVItem, 0, sizeof(MyLVItem));
    memset(&MyNASCCL, 0, sizeof(MyNASCCL));
    memset(sTempBuf, 0, sizeof(sTempBuf));

    ImageList_RemoveAll(LV_MainDlgImageList);
	ListView_DeleteAllItems(FileList_hWnd);

    MyNASCCL.InitializeCryptographySet(sRawKey, RawKeyLen, CRYPTMETHOD);

    lChecksum = MakeChecksum(sRawKey);

    hfSource = CreateFile(
        SrcName,
        GENERIC_READ,
        FILE_SHARE_READ,
        NULL,
        OPEN_EXISTING,
        FILE_ATTRIBUTE_NORMAL,
        NULL
    );

    if(hfSource == INVALID_HANDLE_VALUE)
    {
        sprintf(sStatusText, "Failed to open read file:\r\n'%s'.", SrcName);
        MessageBox(ReadingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
        return false;
    }

    //Read the Application Checksum.
	ReadFile(hfSource, &lReadChecksum1, sizeof(long), &dwBytesRead, NULL);

	//Read the Application Version Checksum.
	ReadFile(hfSource, &dwCompatVersion, sizeof(dwCompatVersion), &dwBytesRead, NULL);

	if(dwCompatVersion != MakeChecksum(COMP_VERSION))
	{
		MessageBox(ReadingDialog_hWnd, "The file you are trying to decrypt was encrypted using an incompatible version.", gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);
	    return false;
	}

	//Read the Password Checksum.
    ReadFile(hfSource, &lReadChecksum2, sizeof(long), &dwBytesRead, NULL);
    MyNASCCL.Decode(&lReadChecksum2, &lReadChecksum2, sizeof(long));

    if(lReadChecksum1 != lChecksum || lReadChecksum2 != HARDCHECKSUM)
    {
        MessageBox(ReadingDialog_hWnd, "Wrong key, Can't decrypt.", gsTitleCaption, MB_ICONINFORMATION|MB_OK|MB_APPLMODAL);
        CloseHandle(hfSource);
        return false;
    }

	//Read the Package Attributes.
    ReadFile(hfSource, &iPkgAttribs, sizeof(iPkgAttribs), &dwBytesRead, NULL);
    MyNASCCL.Decode(&iPkgAttribs, &iPkgAttribs, sizeof(iPkgAttribs));

	//Read the Cryptography Algorithm.
	ReadFile(hfSource, &giCryptAlgo, sizeof(giCryptAlgo), &dwBytesRead, NULL);
    MyNASCCL.Decode(&giCryptAlgo, &giCryptAlgo, sizeof(giCryptAlgo));

	//Read the Julian Creation Date.
	ReadFile(hfSource, &dwCreationDate, sizeof(dwCreationDate), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwCreationDate, &dwCreationDate, sizeof(dwCreationDate));

	//Read the Julian Creation Time.
	ReadFile(hfSource, &dwCreationTime, sizeof(dwCreationTime), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwCreationTime, &dwCreationTime, sizeof(dwCreationTime));

	//File thumb print or serial number.
	ReadFile(hfSource, sThumbPrint, THUMB_PRINT_SZ, &dwBytesRead, NULL);
    MyNASCCL.Decode(sThumbPrint, sThumbPrint, THUMB_PRINT_SZ);

	//Skip past the Reserved header space.
	SetFilePointer(hfSource, RESERVE_HEADER_SZ, NULL, FILE_CURRENT);

	//Read the File count.
	ReadFile(hfSource, &dwFileCount, sizeof(dwFileCount), &dwBytesRead, NULL);
    MyNASCCL.Decode(&dwFileCount, &dwFileCount, sizeof(dwFileCount));
    SendMessage(ReadingStatusBar_hWnd, PBM_SETRANGE32, 0, (LPARAM) dwFileCount);

    //--------------------------------------------------------------------------------

	//dwFilePosition = SetFilePointer(hfSource, NULL, NULL, FILE_CURRENT);
	//ReadFile(hfSource, &usFileNameLen, sizeof(unsigned short), &dwBytesRead, NULL);

    HICON MyIcon = NULL;

    int IconIndexCount = 0;
    DBGICONLIST *MyIcoList;
    MyIcoList = (DBGICONLIST *) calloc(sizeof(DBGICONLIST), dwFileCount + 1);

    MyIcon = LoadIcon(ghAppInstance, MAKEINTRESOURCE(IDI_UNKNOWN));
    ImageList_AddIcon(LV_MainDlgImageList, MyIcon);
    DestroyIcon(MyIcon);
    DeleteObject(MyIcon);

/*
    strcpy(MyIcoList[IconIndexCount].sExt, ".exe");
    strcpy(MyIcoList[IconIndexCount].sDescr, "Application File");
    MyIcoList[IconIndexCount].iIndex = IconIndexCount;
    IconIndexCount++;
*/

    do {
		dwFilePosition = SetFilePointer(hfSource, NULL, NULL, FILE_CURRENT);

		ReadFile(hfSource, &usFileNameLen, sizeof(unsigned short), &dwBytesRead, NULL);

		if(dwBytesRead == 0)
		{
			//This is where this loop ends.
			break;
		}

		if(usFileNameLen <= 0 || usFileNameLen > 255)
        {
            sprintf(sStatusText, "Corrupt data file detected. Error code: %X", usFileNameLen);
            MessageBox(ReadingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

            return false;
        }

		ReadFile(hfSource, sPartFileName, usFileNameLen, &dwBytesRead, NULL);
        MyNASCCL.Decode(sPartFileName, sPartFileName, usFileNameLen);
    	sPartFileName[usFileNameLen] = '\0';
        ReadFile(hfSource, &dwFileSize, sizeof(DWORD), &dwBytesRead, NULL);

        TotalFileSizes = (TotalFileSizes + (((double)dwFileSize) / 1024));

        ReadFile(hfSource, &dwCompressedFileSize, sizeof(DWORD), &dwBytesRead, NULL);

        if(iPkgAttribs &ATTRIB_DATETIME)
		{
			ReadFile(hfSource, &ftCreationTime, sizeof(FILETIME), &dwBytesRead, NULL);
			ReadFile(hfSource, &ftLastWriteTime, sizeof(FILETIME), &dwBytesRead, NULL);
			SetFilePointer(hfSource, dwCompressedFileSize-16, NULL, FILE_CURRENT);
		}
		else SetFilePointer(hfSource, dwCompressedFileSize, NULL, FILE_CURRENT);

		if(iPkgAttribs &ATTRIB_CRCCHECK)
        {
            ReadFile(hfSource, &ulReadCRC, sizeof(unsigned long), &dwBytesRead, NULL);
        }

        _splitpath(sPartFileName, NULL, sDirectory, sFileName, sExt);

        iImageIndex = -1;

		while(iIconSCount < IconIndexCount)
        {
            if( strcmpi(MyIcoList[iIconSCount].sExt, sExt) == 0)
            {
                iImageIndex = MyIcoList[iIconSCount].iIndex;
                strcpy(sDescr, MyIcoList[iIconSCount].sDescr);
				break;
            }
            iIconSCount++;
        }

		if(iImageIndex == -1)
        {
    		char IconFile[255 + 1];
    		int IconIndex = 0;

    		if(GetIconForExt(sExt, IconFile, &IconIndex, sDescr))
            {
                if(strlen(IconFile) > 0)
                {
                    MyIcon = NULL;
					ExtractIconEx(IconFile, IconIndex, NULL, &MyIcon, 1);
                    if(MyIcon)
                    {
                        int xIndex = ImageList_AddIcon(LV_MainDlgImageList, MyIcon);

						if(xIndex != -1)
                        {
                            DestroyIcon(MyIcon);
                            DeleteObject(MyIcon);

                            AllocIconList(&MyIcoList[iIconSCount], strlen(sExt), strlen(sDescr));

                            strcpy(MyIcoList[iIconSCount].sExt, sExt);
                            strcpy(MyIcoList[iIconSCount].sDescr, sDescr);
							MyIcoList[iIconSCount].iIndex = xIndex;

                            iImageIndex = xIndex;

                            IconIndexCount++;
                        }
                    }
                }
            }
        }

        if(iImageIndex == -1)
        {
			sprintf(sDescr, "%s File", strupr(sExt));
			iImageIndex = 0;
		}

        sprintf(sTempBuf, "%s%s", sFileName, sExt);
        MyLVItem.pszText  = sTempBuf;
        if(iPkgAttribs &ATTRIB_DATETIME)
        {
            MyLVItem.mask = LVIF_TEXT|LVIF_IMAGE|LVIF_PARAM;
            MyLVItem.lParam = (LPARAM) ftLastWriteTime.dwHighDateTime;
        }
        else MyLVItem.mask = LVIF_TEXT|LVIF_IMAGE;
        MyLVItem.iImage   = iImageIndex;
        MyLVItem.iSubItem = 0;
        ListView_InsertItem(FileList_hWnd, &MyLVItem);

        MyLVItem.pszText  = sDescr; // Type
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 1;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        if(iPkgAttribs &ATTRIB_DATETIME)
        {
            sprintf(sTempBuf, "%x", ulReadCRC);

            char TimeTemp[64];
            char DateTemp[64];

            SYSTEMTIME MyST;
            FILETIME lFT;
            FileTimeToLocalFileTime(&ftLastWriteTime, &lFT);
            if(FileTimeToSystemTime(&lFT, &MyST))
            {
                GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &MyST, NULL, TimeTemp, 64);
                GetDateFormat(NULL, NULL, &MyST, "MM-dd-yyyy", DateTemp, 64);
                sprintf(sTempBuf, "%s %s", DateTemp, TimeTemp);
            }
            else strcpy(sTempBuf, "(n/a)");
        }
        else strcpy(sTempBuf, "(n/a)");

        MyLVItem.pszText  = sTempBuf; // Modified
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 2;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        sprintf(sTempBuf, "%d", dwFileSize);
        MyLVItem.pszText  = FormatiString(TempBuf0, sTempBuf); // Size
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 3;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        if(iPkgAttribs &ATTRIB_COMPRESSION)
        {
            if(dwCompressedFileSize != 0 && dwFileSize != 0 && dwFileSize > dwCompressedFileSize)
            {
                float fCompressedFileSize = (float) dwCompressedFileSize;
                float fFileSize = (float) dwFileSize;
                float Ratio = 100 - ((fCompressedFileSize/fFileSize) * 100);
                sprintf(sTempBuf, "%.2f%%", Ratio);
            }
            else strcpy(sTempBuf, "0.00%");
        }
        else strcpy(sTempBuf, "(n/a)");
        MyLVItem.pszText  = sTempBuf; // Ratio
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 4;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        sprintf(sTempBuf, "%d", dwCompressedFileSize);
        MyLVItem.pszText  = FormatiString(TempBuf0, sTempBuf); // Packed
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 5;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        if(iPkgAttribs &ATTRIB_CRCCHECK)
        {
            sprintf(sTempBuf, "%x", ulReadCRC);
        }
        else strcpy(sTempBuf, "(n/a)");
        MyLVItem.pszText  = sTempBuf; // CRC
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 6;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        if(iPkgAttribs &ATTRIB_ATTRIBUTES)
        {
            strcpy(sTempBuf, "A");
        }
        else strcpy(sTempBuf, "(n/a)");
        MyLVItem.pszText  = sTempBuf; // Attributes
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 7;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        if(iPkgAttribs &ATTRIB_SAVEPATHS)
        {
            strcpy(sTempBuf, sDirectory);
        }
        else strcpy(sTempBuf, "(n/a)");
        MyLVItem.pszText  = sTempBuf; // Path
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 8;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        sprintf(sTempBuf, "%d", dwFilePosition);
        MyLVItem.pszText  = sTempBuf; // Position
        MyLVItem.mask     = LVIF_TEXT;
        MyLVItem.iSubItem = 9;
        ListView_SetItem(FileList_hWnd, &MyLVItem);

        dwFileCountLoop++;
        dwPackageProgress = dwFileCountLoop;

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Sleep(1000);
            break;
        }
    } while(dwBytesRead != 0 && !(usFileNameLen <= 0 || usFileNameLen > 255));

    CloseHandle(hfSource);


    sprintf(sTempBuf, "%d", dwFileCount);
    FormatiString(TempBuf0, sTempBuf);

    sprintf(sTempBuf, "%.0f", TotalFileSizes);
    FormatiString(TempBuf1, sTempBuf);

    sprintf(sStatusText, "Total %s files, %s KB", TempBuf0, TempBuf1);
    SendMessage(StatusBar_hWnd, (UINT)SB_SETTEXT, (WPARAM)1, (LPARAM)sStatusText);
    SendMessage(StatusBar_hWnd, (UINT)SB_SETTEXT, (WPARAM)0, (LPARAM)"Ready");

    // Free the icon search list.
    iIconSCount = 0;
    while(iIconSCount < IconIndexCount)
    {
        FreeIconList(&MyIcoList[iIconSCount]);
		iIconSCount++;
	}

    if(usFileNameLen <= 0 || usFileNameLen > 255)
    {
        sprintf(sStatusText, "Corrupt data File detected. Error code: %X", usFileNameLen);
        MessageBox(ReadingDialog_hWnd, sStatusText, gsTitleCaption, MB_ICONEXCLAMATION|MB_OK|MB_APPLMODAL);

        return false;
    }

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI ExtractEntireArchive_Worker_Thread(LPVOID xData)
{
    char *CmdLine = (char *) xData;

	WaitOnWindow(PackingDialog_hWnd);

    Set_Text(PackingActionText_hWnd, "Starting process...");

    Sleep(100);

	FixDirTarget(gsExtractLocation);

	if(BreakPackage(CmdLine, gsExtractLocation, gsPassword, strlen(gsPassword)))
    {
        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(PackingActionText_hWnd, "Cancelled.");
            Set_Text(PackingStatusText_hWnd, "");
            Sleep(1000);
        }
        else{
            Set_Text(PackingActionText_hWnd, "Done.");
            Set_Text(PackingStatusText_hWnd, "");
            Sleep(100);
        }

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;
        EndDialog(PackingDialog_hWnd, 1);

        return 0;
    }
    else{
        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(PackingActionText_hWnd, "Cancelled.");
            Set_Text(PackingStatusText_hWnd, "");
        }

        Sleep(1000);

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;
        EndDialog(PackingDialog_hWnd, 1);

        return 0;
    }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI ViewExisting_Worker_Thread(LPVOID xData)
{
    char *CmdLine = (char *) xData;

	WaitOnWindow(ReadingDialog_hWnd);

    Set_Text(ReadingStatusText_hWnd, "Please wait... Decrypting the archive.");

	Sleep(100);

	gbArchiveOpen = false;
    SendMessage(MainDialog_hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)gsTitleCaption);

    if(PopGrid(CmdLine, gsPassword, strlen(gsPassword)))
    {
        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
			Set_Text(ReadingStatusText_hWnd, "Cancelled.");
            Sleep(1000);
            dwPackageProgress = 0;
            gdwCurrentJobType = 0;
            EndDialog(ReadingDialog_hWnd, 0);
        }
        else{
			char sCaption[MAX_PATH + 100];
			char sFileName[MAX_PATH + 100];

			_splitpath(CmdLine, NULL, NULL, sFileName, NULL);

			sprintf(sCaption, "%s [%s]", gsTitleCaption, sFileName);
	        SendMessage(MainDialog_hWnd, (UINT)WM_SETTEXT, (WPARAM)0, (LPARAM)sCaption);
			
			Set_Text(ReadingStatusText_hWnd, "Done.");
            Sleep(250);
			gbArchiveOpen = true;
            dwPackageProgress = 0;
            gdwCurrentJobType = 0;
            EndDialog(ReadingDialog_hWnd, 1);
        }

        return 0;
    }
    else{
        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(ReadingStatusText_hWnd, "Cancelled.");
        }

		Sleep(1000);
        dwPackageProgress = 0;
        gdwCurrentJobType = 0;
        EndDialog(ReadingDialog_hWnd, 0);

        return 0;
    }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI CreateNewDirArchive_Worker_Thread(LPVOID xData)
{
    char *CmdLine = (char *) xData;

	WaitOnWindow(PackingDialog_hWnd);

    Set_Text(PackingStatusText_hWnd, "Please wait...");

    Sleep(100);

    MKPKGPARAM MyMkPkgParam;
	memset(&MyMkPkgParam, 0, sizeof(MyMkPkgParam));
	MyMkPkgParam.sSrcFolder  = CmdLine;
	MyMkPkgParam.sTarName    = gsWorkingOutput;
	MyMkPkgParam.sRawKey     = gsPassword;
	MyMkPkgParam.iRawKeyLen  = strlen(gsPassword);
	MyMkPkgParam.iPkgAttribs = giPkgAttribs;
	MyMkPkgParam.iOptions    = 0;

	if(MakePackage(&MyMkPkgParam))
    {
        strcpy(gsArchiveName, gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(PackingStatusText_hWnd, "Cancelled.");
            Sleep(1000);
            EndDialog(PackingDialog_hWnd, 0);
            DeleteFile(gsWorkingOutput);
        }
        else{
            Set_Text(PackingStatusText_hWnd, "Done.");
            Sleep(1000);
            EndDialog(PackingDialog_hWnd, 1);
        }

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        return 0;
    }
    else{
        DeleteFile(gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
            Set_Text(PackingStatusText_hWnd, "Cancelled.");

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        Sleep(1000);
        EndDialog(PackingDialog_hWnd, 0);

        return 0;
    }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI CreateFileArchive_Worker_Thread(LPVOID xData)
{
	char *CmdLine = (char *) xData;

	WaitOnWindow(PackingDialog_hWnd);
	
    Set_Text(PackingStatusText_hWnd, "Please wait...");

    Sleep(100);

    MKPKGPARAM MyMkPkgParam;
	memset(&MyMkPkgParam, 0, sizeof(MyMkPkgParam));
	MyMkPkgParam.sSrcFolder  = CmdLine;
	MyMkPkgParam.sTarName    = gsWorkingOutput;
	MyMkPkgParam.sRawKey     = gsPassword;
	MyMkPkgParam.iRawKeyLen  = strlen(gsPassword);
	MyMkPkgParam.iPkgAttribs = giPkgAttribs;
	MyMkPkgParam.iOptions    = ATTRIB_USEALTFILEPACK;

	if(MakePackage(&MyMkPkgParam))
    {
        strcpy(gsArchiveName, gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(PackingStatusText_hWnd, "Cancelled.");
            Sleep(1000);
            EndDialog(PackingDialog_hWnd, 0);
            DeleteFile(gsWorkingOutput);
        }
        else{
            Set_Text(PackingStatusText_hWnd, "Done.");
            Sleep(1000);
            EndDialog(PackingDialog_hWnd, 1);
        }

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        return 0;
    }
    else{
        DeleteFile(gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
            Set_Text(PackingStatusText_hWnd, "Cancelled.");

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        Sleep(1000);
        EndDialog(PackingDialog_hWnd, 0);

        return 0;
    }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI AddFiles_Worker_Thread(LPVOID xData)
{
	char *CmdLine = (char *) xData;

	WaitOnWindow(PackingDialog_hWnd);
	
    Set_Text(PackingStatusText_hWnd, "Please wait...");

    Sleep(100);

    MKPKGPARAM MyMkPkgParam;
	memset(&MyMkPkgParam, 0, sizeof(MyMkPkgParam));
	MyMkPkgParam.sSrcFolder  = CmdLine;
	MyMkPkgParam.sTarName    = gsWorkingOutput;
	MyMkPkgParam.sRawKey     = gsPassword;
	MyMkPkgParam.iRawKeyLen  = strlen(gsPassword);
	MyMkPkgParam.iPkgAttribs = giPkgAttribs;
	MyMkPkgParam.iOptions    = ATTRIB_USEALTFILEPACK|ATTRIB_ADDFILES;

	if(MakePackage(&MyMkPkgParam))
    {
        strcpy(gsArchiveName, gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
        {
            Set_Text(PackingStatusText_hWnd, "Cancelled.");
            Sleep(1000);
            EndDialog(PackingDialog_hWnd, 0);
            /*/
			//We do not want to delete this file.
			//DeleteFile(gsWorkingOutput);
			/*/
        }
        else{
            Set_Text(PackingStatusText_hWnd, "Done.");
            Sleep(100);
            EndDialog(PackingDialog_hWnd, 1);
        }

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        return 0;
    }
    else{
        DeleteFile(gsWorkingOutput);

        if(gdwCurrentJobType == JOB_TYPE_CANCEL)
            Set_Text(PackingStatusText_hWnd, "Cancelled.");

        dwPackageProgress = 0;
        gdwCurrentJobType = 0;

        Sleep(1000);
        EndDialog(PackingDialog_hWnd, 0);

        return 0;
    }

	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
